@@ -6,15 +6,13 @@
 //!
 //! [`process_minidump`]: fn.process_minidump.html
 
-mod dwarf_symbolizer;
 mod evil;
 mod process_state;
 mod processor;
 mod stackwalker;
-mod symbols;
+pub mod symbols;
 mod system_info;
 
-pub use crate::dwarf_symbolizer::DwarfSymbolizer;
 pub use crate::process_state::*;
 pub use crate::processor::*;
 pub use crate::stackwalker::*;
