@@ -389,6 +389,17 @@ impl ProcessState {
     /// This is very verbose, it implements the output format used by
     /// minidump_stackwalk.
     pub fn print<T: Write>(&self, f: &mut T) -> io::Result<()> {
+        self.print_internal(f, false)
+    }
+
+    /// Write a brief human-readable description of the process state to `f`.
+    ///
+    /// Only includes the summary at the top and a backtrace of the crashing thread.
+    pub fn print_brief<T: Write>(&self, f: &mut T) -> io::Result<()> {
+        self.print_internal(f, true)
+    }
+
+    fn print_internal<T: Write>(&self, f: &mut T, brief: bool) -> io::Result<()> {
         writeln!(f, "Operating system: {}", self.system_info.os.long_name())?;
         if let Some(ref ver) = self.system_info.os_version {
             writeln!(f, "                  {}", ver)?;
@@ -486,6 +497,12 @@ Crash address: {:#x}
             stack.print(f)?;
             writeln!(f)?;
         }
+
+        // We're done if this is a brief report!
+        if brief {
+            return Ok(());
+        }
+
         for (i, stack) in self.threads.iter().enumerate() {
             if eq_some(self.requesting_thread, i) {
                 // Don't print the requesting thread again,
@@ -587,14 +604,18 @@ Unknown streams encountered:
     }
 
     /// Outputs json in a schema compatible with mozilla's Socorro crash reporting servers.
+    ///
+    #[doc = include_str!("../json-schema.md")]
     pub fn print_json<T: Write>(&self, f: &mut T, pretty: bool) -> Result<(), serde_json::Error> {
+        // See ../json-schema.md for details on this format.
+
         let sys = &self.system_info;
 
         // Curry self for use in `map`
         let json_hex = |val: u64| -> String { self.json_hex(val) };
 
         let mut output = json!({
-            // TODO: I guess we should still produce some JSON in some failure modes?
+            // Currently unused, we either produce no output or successful output.
             // OK | ERROR_* | SYMBOL_SUPPLIER_INTERRUPTED
             "status": "OK",
             "system_info": {
@@ -605,7 +626,6 @@ Unknown streams encountered:
                 "cpu_arch": sys.cpu.to_string(),
                 "cpu_info": sys.cpu_info,
                 "cpu_count": sys.cpu_count,
-                // TODO: Issue #19
                 // optional
                 "cpu_microcode_version": sys.cpu_microcode_version,
             },
@@ -684,10 +704,6 @@ Unknown streams encountered:
             "thread_count": self.threads.len(),
             "threads": self.threads.iter().map(|thread| json!({
                 "frame_count": thread.frames.len(),
-                "frames_truncated": false,
-                // optional, if truncated, this is the original total
-                "total_frames": thread.frames.len(),
-                // TODO: Issue #156
                 // optional
                 "last_error_value": thread.last_error_value.map(|error| error.to_string()),
                 // optional
@@ -737,7 +753,7 @@ Unknown streams encountered:
 
         if let Some(requesting_thread) = self.requesting_thread {
             // Copy the crashing thread into a top-level "crashing_thread" field and:
-            // * Add a "thread_index" field to indicate which thread it was
+            // * Add a "threads_index" field to indicate which thread it was
             // * Add a "registers" field to its first frame
             //
             // Note that we currently make crashing_thread a strict superset
@@ -758,7 +774,7 @@ Unknown streams encountered:
             let frame = frames[0].as_object_mut().unwrap();
 
             frame.insert(String::from("registers"), registers);
-            thread_obj.insert(String::from("thread_index"), json!(requesting_thread));
+            thread_obj.insert(String::from("threads_index"), json!(requesting_thread));
 
             output
                 .as_object_mut()
