@@ -1,3 +1,98 @@
+//! This module defines the interface between minidump-processor and its [Symbolizer][].
+//!
+//! There can only be one [Symbolizer][], and this is configured by minidump-processor's Cargo
+//! feature flags. The currently defined Symbolizers are:
+//!
+//! * breakpad_symbols -- feature: breakpad-syms (currently the default)
+//! * symbolic -- feature: symbolic-syms (not yet implemented, but compiles)
+//!
+//!
+//! minidump-processor and the [Symbolizer][] communicate using a series of traits. The symbolizer
+//! must provide implementations of these traits:
+//!
+//! * [SymbolProvider][] - provides symbolication, cfi evaluation, and debug statistics
+//!     * Implemented by [Symbolizer][]
+//!     * This actually doesn't need to be a trait in the current design, it exists to allow
+//!       multiple symbolicators to be used together, via [MultiSymbolProvider][]. The other
+//!       SymbolProviders have been removed, but I figured it would be a waste to throw out
+//!       this minimally intrusive machinery.
+//!
+//! * [SymbolSupplier][] - maps a [Module][] to a [SymbolFile][]
+//!     * minidump-processor does not directly use this, it's just there so the Symbolizer can
+//!       generically handle different symbol fetching strategies (which minidump-processor
+//!       selects and configures).
+//!
+//!
+//!
+//! While minidump-processor provides implementations of these traits:
+//!
+//! * [FrameSymbolizer][] - callbacks that symbolication uses to return its results.
+//!     * Implemented by [StackFrame][crate::process_state::StackFrame]
+//!     * Implemented by DummyFrame (private, for a stack scanning heuristic)
+//! * [FrameWalker][] - callbacks that cfi eval uses to read callee state and write caller state.
+//!     * Implemented by CfiStackWalker (private)
+//!
+//!
+//!
+//! The symbolizer is responsible for providing the following concrete functions, which
+//! minidump-processor uses to select and configure the symbol fetching strategy:
+//!
+//! * [http_symbol_supplier][] - a [SymbolSupplier][] that can find symbols over HTTP (and cache).
+//! * [simple_symbol_supplier][] - a [SymbolSupplier][] that can find symbols on disk.
+//! * [string_symbol_supplier][] - a mock [SymbolSupplier][] for tests.
+//!
+//!
+//!
+//! And the following concrete types:
+//!
+//! * [Symbolizer][] - the main interface of the symbolizer, implementing [SymbolProvider][].
+//!     * Wraps the [SymbolSupplier][] implementation that minidump-processor selects.
+//!     * Queries the [SymbolSupplier] and manages the SymbolFiles however it pleases.
+//! * [SymbolStats][] - debug statistic output.
+//! * [SymbolFile][] - a payload that a [SymbolProvider][] returns to the Symbolizer.
+//!     * Never handled by minidump-processor, public for the trait. (use this for whatever)
+//! * [SymbolError][] - possible errors a [SymbolProvider][] can yield.
+//!     * Never handled by minidump-processor, public for the trait. (use this for whatever)
+//! * [FillSymbolError][] - possible errors for `fill_symbol`.
+//!     * While this *is* handled by minidump-processor, it doesn't actually look at the value. It's
+//!       just there to be An Error Type for the sake of API design.
+//!
+//!
+//!
+//! # Example
+//!
+//! ```rust
+//! use minidump::Minidump;
+//! use minidump_processor::{http_symbol_supplier, ProcessorOptions, Symbolizer};
+//!
+//! fn main() -> Result<(), ()> {
+//!     // Read the minidump
+//!     let dump = Minidump::read_path("../testdata/test.dmp").map_err(|_| ())?;
+//!  
+//!     // Configure the symbolizer and processor
+//!     let symbols_urls = vec![String::from("https://symbols.totallyrealwebsite.org")];
+//!     let symbols_paths = vec![];
+//!     let mut symbols_cache = std::env::temp_dir();
+//!     symbols_cache.push("minidump-cache");
+//!     let symbols_tmp = std::env::temp_dir();
+//!     let timeout = std::time::Duration::from_secs(1000);
+//!  
+//!     let options = ProcessorOptions::default();
+//!     let provider = Symbolizer::new(http_symbol_supplier(
+//!         symbols_paths,
+//!         symbols_urls,
+//!         symbols_cache,
+//!         symbols_tmp,
+//!         timeout,
+//!     ));
+//!  
+//!     let state = minidump_processor::process_minidump_with_options(&dump, &provider, options)
+//!         .map_err(|_| ())?;
+//!     state.print(&mut std::io::stdout()).map_err(|_| ())?;
+//!     Ok(())
+//! }
+//! ```
+//!
 use minidump::Module;
 use std::collections::HashMap;
 pub use symbols_shim::*;
@@ -69,11 +164,13 @@ impl SymbolProvider for MultiSymbolProvider {
 mod symbols_shim {
     use super::SymbolProvider;
     pub use breakpad_symbols::{
-        FillSymbolError, FrameSymbolizer, FrameWalker, SymbolStats, SymbolSupplier, Symbolizer,
+        FillSymbolError, FrameSymbolizer, FrameWalker, SymbolError, SymbolFile, SymbolStats,
+        SymbolSupplier, Symbolizer,
     };
     use minidump::Module;
     use std::collections::HashMap;
     use std::path::PathBuf;
+    use std::time::Duration;
     impl SymbolProvider for Symbolizer {
         fn fill_symbol(
             &self,
@@ -92,22 +189,47 @@ mod symbols_shim {
 
     /// Gets a SymbolSupplier that looks up symbols by path or with urls.
     ///
-    /// May use the `symbols_cache` path to store downloads.
+    /// * `symbols_paths` is a list of paths to check for symbol files. Paths
+    ///   are searched in order until one returns a payload. If none do, then
+    ///   urls are used.
+    ///
+    /// * `symbols_urls` is a list of "base urls" that should all point to Tecken
+    ///   servers. urls are queried in order until one returns a payload. If none
+    ///   do, then it's an error.
+    ///
+    /// * `symbols_cache` is a directory where an on-disk cache should be located.
+    ///   This should be assumed to be a "temp" directory that another process
+    ///   you don't control is garbage-collecting old files from (to provide an LRU cache).
+    ///   The cache is queried before paths and urls (otherwise it wouldn't be much of a cache).
+    ///   
+    /// * `symbols_tmp` is a directory where symbol files should be downloaded to
+    ///   before atomically swapping them into the cache. Has the same "temp"
+    ///   assumptions as symbols_cache.
+    ///
+    /// * `timeout` a maximum time limit for a symbol file download. This
+    ///   is primarily defined to avoid getting stuck on buggy infinite downloads.
+    ///   As of this writing, minidump-stackwalk defaults this to 1000 seconds. In
+    ///   the event of a timeout, the supplier may still try to parse the truncated
+    ///   download.
     pub fn http_symbol_supplier(
         symbol_paths: Vec<PathBuf>,
         symbol_urls: Vec<String>,
         symbols_cache: PathBuf,
         symbols_tmp: PathBuf,
+        timeout: Duration,
     ) -> impl SymbolSupplier {
         breakpad_symbols::HttpSymbolSupplier::new(
             symbol_urls,
             symbols_cache,
             symbols_tmp,
             symbol_paths,
+            timeout,
         )
     }
 
     /// Gets a SymbolSupplier that looks up symbols by path.
+    ///
+    /// Paths are queried in order until one returns a payload.
     pub fn simple_symbol_supplier(symbol_paths: Vec<PathBuf>) -> impl SymbolSupplier {
         breakpad_symbols::SimpleSymbolSupplier::new(symbol_paths)
     }
@@ -127,6 +249,7 @@ mod symbols_shim {
     use minidump::Module;
     use std::collections::HashMap;
     use std::path::PathBuf;
+    use std::time::Duration;
 
     // Import symbolic here
 
@@ -171,10 +294,6 @@ mod symbols_shim {
         fn set_ra(&mut self, val: u64) -> Option<()>;
     }
 
-    /// Possible results of locating symbols. (can be opaque, not used externally)
-    #[derive(Debug)]
-    pub struct SymbolResult;
-
     /// Symbolicate stack frames.
     ///
     /// A `Symbolizer` manages loading symbols and looking up symbols in them
@@ -219,45 +338,48 @@ mod symbols_shim {
         fn walk_frame(&self, _module: &dyn Module, _walker: &mut dyn FrameWalker) -> Option<()> {
             unimplemented!()
         }
-    }
-
-    pub struct HttpSymbolSupplier {}
-
-    pub struct SimpleSymbolSupplier {}
-
-    pub struct StringSymbolSupplier {}
-
-    impl SymbolSupplier for HttpSymbolSupplier {
-        fn locate_symbols(&self, _module: &dyn Module) -> Result<SymbolFile, SymbolError> {
-            unimplemented!()
-        }
-    }
-
-    impl SymbolSupplier for SimpleSymbolSupplier {
-        fn locate_symbols(&self, _module: &dyn Module) -> Result<SymbolFile, SymbolError> {
-            unimplemented!()
-        }
-    }
-
-    impl SymbolSupplier for StringSymbolSupplier {
-        fn locate_symbols(&self, _module: &dyn Module) -> Result<SymbolFile, SymbolError> {
+        fn stats(&self) -> HashMap<String, SymbolStats> {
             unimplemented!()
         }
     }
 
     /// Gets a SymbolSupplier that looks up symbols by path or with urls.
     ///
-    /// May use the `symbols_cache` path to store downloads.
+    /// * `symbols_paths` is a list of paths to check for symbol files. Paths
+    ///   are searched in order until one returns a payload. If none do, then
+    ///   urls are used.
+    ///
+    /// * `symbols_urls` is a list of "base urls" that should all point to Tecken
+    ///   servers. urls are queried in order until one returns a payload. If none
+    ///   do, then it's an error.
+    ///
+    /// * `symbols_cache` is a directory where an on-disk cache should be located.
+    ///   This should be assumed to be a "temp" directory that another process
+    ///   you don't control is garbage-collecting old files from (to provide an LRU cache).
+    ///   The cache is queried before paths and urls (otherwise it wouldn't be much of a cache).
+    ///   
+    /// * `symbols_tmp` is a directory where symbol files should be downloaded to
+    ///   before atomically swapping them into the cache. Has the same "temp"
+    ///   assumptions as symbols_cache.
+    ///
+    /// * `timeout` a maximum time limit (in seconds) for a symbol file download. This
+    ///   is primarily defined to avoid getting stuck on buggy infinite downloads.
+    ///   As of this writing, minidump-stackwalk defaults this to 1000 seconds. In
+    ///   the event of a timeout, the supplier may still try to parse the truncated
+    ///   download.
     pub fn http_symbol_supplier(
         _symbol_paths: Vec<PathBuf>,
         _symbol_urls: Vec<String>,
         _symbols_cache: PathBuf,
         _symbols_tmp: PathBuf,
+        _timeout: Duration,
     ) -> impl SymbolSupplier {
         HttpSymbolSupplier {}
     }
 
     /// Gets a SymbolSupplier that looks up symbols by path.
+    ///
+    /// Paths are queried in order until one returns a payload.
     pub fn simple_symbol_supplier(_symbol_paths: Vec<PathBuf>) -> impl SymbolSupplier {
         SimpleSymbolSupplier {}
     }
@@ -295,7 +417,7 @@ mod symbols_shim {
         /// In this case other symbol providers may still be able to find it!
         NotFound,
         /// Symbol file could not be loaded into memory.
-        LoadError(Error),
+        LoadError,
         /// Symbol file was too corrupt to be parsed at all.
         ///
         /// Because symbol files are pretty modular, many corruptions/ambiguities
@@ -303,14 +425,14 @@ mod symbols_shim {
         /// (e.g. a bad STACK WIN line can be discarded without affecting anything
         /// else). But sometimes we can't make any sense of the symbol file, and
         /// you find yourself here.
-        ParseError(Error),
+        ParseError,
     }
 
     #[derive(Debug)]
     pub struct FillSymbolError {}
 
     // Whatever representation you want, rust-minidump won't look at it.
-    struct SymbolFile {}
+    pub struct SymbolFile {}
 
     /// Statistics on the symbols of a module.
     #[derive(Default, Debug)]
@@ -322,4 +444,31 @@ mod symbols_shim {
         /// If we tried to parse the symbols, but failed.
         pub corrupt_symbols: bool,
     }
+
+    // These suppliers are entriely private to the implementation, so do whatever you
+    // want with them.
+
+    struct HttpSymbolSupplier {}
+
+    struct SimpleSymbolSupplier {}
+
+    struct StringSymbolSupplier {}
+
+    impl SymbolSupplier for HttpSymbolSupplier {
+        fn locate_symbols(&mut self, _module: &dyn Module) -> Result<SymbolFile, SymbolError> {
+            unimplemented!()
+        }
+    }
+
+    impl SymbolSupplier for SimpleSymbolSupplier {
+        fn locate_symbols(&mut self, _module: &dyn Module) -> Result<SymbolFile, SymbolError> {
+            unimplemented!()
+        }
+    }
+
+    impl SymbolSupplier for StringSymbolSupplier {
+        fn locate_symbols(&mut self, _module: &dyn Module) -> Result<SymbolFile, SymbolError> {
+            unimplemented!()
+        }
+    }
 }
